/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.utils;

import java.net.URLEncoder;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;
import java.util.TimeZone;

import javax.servlet.http.HttpServletRequest;

import com.ibm.rfid.premises.logger.Logger;
import com.ibm.sensorevent.model.IBMSensorEvent;
import com.ibm.sensorevent.model.ISensorEvent;
import com.ibm.sensorevent.model.generic.SensorEventException;
import com.ibm.sensorevent.util.Constants;
import com.ibm.sensorevent.util.SIBusPublisher;

/**
 * Common UI utils
 */
public class Utils {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

	private static Utils mySingleton;
	public CTLogger logger;
	public ResourceLocator locator;
	
	public static final int DEFAULT_MAX_EVENTS = 1000;
	public static final String EVENT_SOURCEID = "ibmct";
	
	public static final String JDBC_SESSION_FACADE_REF = "java:comp/env/ejb/JdbcSessionFacadeRef";
	public static final String RUC_SESSION_FACADE_REF = "java:comp/env/ejb/RucSessionFacadeRef";
	public static final String PREMISES_SESSION_FACADE_REF = "java:comp/env/ejb/PremisesSessionFacadeRef";
	
	public static final String GRAI_96 = "urn:epc:tag:grai-96:";
	public static final String RAW_96 = "urn:epc:raw:96.x";
	
	protected Utils() {
		try {
			this.logger = (CTLogger) CTLogger.singleton();
			this.logger.traceEntry(this, "Utils");
			this.locator = ResourceLocator.singleton();
		} catch (Exception e) {
			e.printStackTrace();
			try {
				this.logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "Utils", e);
			} catch (Exception e2) {
				// no-op
			}
		} finally {
			try {
				this.logger.traceExit(this, "Utils");
			} catch (Exception e) {
				// no-op
			}
		}
	}
	
	/**
	 * Returns an initialized Utils instance
	 * @return Utils
	 */
	public synchronized static Utils singleton() {
		if (mySingleton == null) {
			mySingleton = new Utils();
		}
		
		return mySingleton;
	}
	
	/**
	 * Returns a UserRoles instance for the current request
	 * @param request
	 * @return UserRoles
	 * @throws Exception
	 */
	public UserRoles getUserRoles(HttpServletRequest request) throws Exception {
		return new UserRoles(request);
	}
	
	/**
	 * Returns message with value1 substituted for all "{0}"
	 * @param message
	 * @param value1
	 * @return String
	 */
	public String formatMessage(String message, Object value1) {
		return this.formatMessage(message, new Object[] {value1});
	}
	
	/**
	 * Returns message with value1 substituted for all "{0}"
	 * and value2 for all "{1}"
	 * @param message
	 * @param value1
	 * @param value2
	 * @return String
	 */
	public String formatMessage(String message, Object value1, Object value2) {
		return this.formatMessage(message, new Object[] {value1, value2});
	}
	
	/**
	 * Returns message with value1 substituted for all "{0}",
	 * value2 for all "{1}", and value3 for all "{2}"
	 * @param message
	 * @param value1
	 * @param value2
	 * @param value3
	 * @return String
	 */
	public String formatMessage(String message, Object value1, Object value2, Object value3) {
		return this.formatMessage(message, new Object[] {value1, value2, value3});
	}
	
	/**
	 * Returns message with elements from array values substituted; 
	 * for example, substitute values[0] for all "{0}", values[1]
	 * for all "{1}", etc.
	 * @param message
	 * @param values
	 * @return String
	 */
	public String formatMessage(String message, Object[] values) {
		MessageFormat messageFormat = new MessageFormat(message);
		String result = messageFormat.format(values);
		return result;
	}
	
	public void publishEvent(String eventType, String topic, ISensorEvent event) throws Exception {
		SIBusPublisher publisher = new SIBusPublisher();
		publisher.publish(eventType, topic, event);
	}
	
	public void publishDebugEvent(String s) throws Exception {
		Map<String, String> m = new HashMap<String, String>();
		m.put("value", s);
		
		String eventType = Constants.Alert_Debug_Event_Name;
		ISensorEvent event = IBMSensorEvent.getInstance(eventType, m);
		event.getHeader().setSourceId(EVENT_SOURCEID);
		
		String topic = Constants.IBMSENSOREVENT_TOPIC_SPACE_NAME + "/" + EVENT_SOURCEID + "/" + eventType;
		
		this.publishEvent(eventType, topic, event);
	}
	
	public void publishInfoEvent(String s) throws Exception {
		Map<String, String> m = new HashMap<String, String>();
		m.put("value", s);
		
		String eventType = Constants.Alert_Info_Event_Name;
		ISensorEvent event = IBMSensorEvent.getInstance(eventType, m);
		event.getHeader().setSourceId(EVENT_SOURCEID);
		
		String topic = Constants.IBMSENSOREVENT_TOPIC_SPACE_NAME + "/" + EVENT_SOURCEID + "/" + eventType;
		
		this.publishEvent(eventType, topic, event);
	}
	
	public void publishWarningEvent(String s) throws Exception {
		Map<String, String> m = new HashMap<String, String>();
		m.put("value", s);
		
		String eventType = Constants.Alert_Warning_Event_Name;
		ISensorEvent event = IBMSensorEvent.getInstance(eventType, m);
		event.getHeader().setSourceId(EVENT_SOURCEID);
		
		String topic = Constants.IBMSENSOREVENT_TOPIC_SPACE_NAME + "/" + EVENT_SOURCEID + "/" + eventType;
		
		this.publishEvent(eventType, topic, event);
	}
	
	public void publishErrorEvent(String s) throws Exception {
		Map<String, String> m = new HashMap<String, String>();
		m.put("value", s);
		
		String eventType = Constants.Alert_Error_Event_Name;
		ISensorEvent event = IBMSensorEvent.getInstance(eventType, m);
		event.getHeader().setSourceId(EVENT_SOURCEID);
		
		String topic = Constants.IBMSENSOREVENT_TOPIC_SPACE_NAME + "/" + EVENT_SOURCEID + "/" + eventType;
		
		this.publishEvent(eventType, topic, event);
	}
	
	/**
	 * Java equivalent of JavaScript's escape function; basically, this is a
	 * URLEncoder.encode followed by changing all "+" to "%20".
	 * @param s
	 * @return
	 */
	public String escape(String s) {
		try {
			String result = URLEncoder.encode(s, "UTF-8").replaceAll("\\+", "%20");
			return result;
		} catch (Exception e) {
			return s;
		}
	}
	
	/**
	 * Return the result of x / y to 2 decimal places (e.g. 0.12)
	 * @param x
	 * @param y
	 * @return
	 */
	public double percentage(double x, double y) {
		return percentage(x, y, 2);
	}
	
	/**
	 * Return the result of x / y to the given number of decimal places (e.g 0.123
	 * when decimalPlaces is set to 3)
	 * @param x
	 * @param y
	 * @param decimalPlaces
	 * @return
	 */
	public double percentage(double x, double y, int decimalPlaces) {
		try {
			double p = Math.round(x / y * Math.pow(10, decimalPlaces)) / Math.pow(10, decimalPlaces);
			return p;
		} catch (Exception e) {
			return Double.NaN;
		}
	}
	
	/**
	 * Return the result of x / y to 2 decimal places as a percentage (e.g. 12.34%)
	 * @param x
	 * @param y
	 * @return
	 */
	public String percentageAsString(double x, double y) {
		return percentageAsString(x, y, 2);
	}
	
	/**
	 * Return the result of x / y to the given number of decimal places as a percentage (e.g. 12.345%
	 * when decimalPlaces is set to 3)
	 * @param x
	 * @param y
	 * @param decimalPlaces
	 * @return
	 */
	public String percentageAsString(double x, double y, int decimalPlaces) {
		try {
			double p = Math.round(x / y * 100D * Math.pow(10, decimalPlaces)) / Math.pow(10, decimalPlaces);
			return p + "%";
		} catch (Exception e) {
			return null;
		}	
	}
	
	/**
	 * Returns the current date formatted as "yyyy-MM-dd hh:mm:ss z"
	 * @return
	 */
	public String currentDateAsString() {
		return currentDateAsString("yyyy-MM-dd hh:mm:ss z");
	}
	
	/** 
	 * Returns the current date formatted using the given format string (see java.util.SimpleDateFormat
	 * for allowed format strings)
	 * @param format
	 * @return
	 */
	public String currentDateAsString(String format) {
		Calendar cal = GregorianCalendar.getInstance();
		return this.dateAsString(cal.getTime(), format);
	}
	
	/**
	 * Returns the date formatted using the format string
	 * @param d
	 * @param format
	 * @return
	 */
	public String dateAsString(Date d, String format) {
		SimpleDateFormat df = new SimpleDateFormat(format);
		return df.format(d.getTime());
	}
	
	/**
	 * Converts the given ISO 8601 date string into a java.util.Date.
	 * 
	 * <P>ISO 8601 date strings have the form <pre>yyyy-MM-ddTHH:mm:ss.MMMZ</pre>
	 * 
	 * @param s The string to convert
	 * @return java.util.Date
	 */
	public Date parseISO8601Date(String s) {
		String year = null;
		String month = null;
		String day = null;
		String hour = null;
		String minute = null;
		String seconds = null;
		String millis = null;
		Calendar calendar = null;

		if (s == null) {
			return null;
		}
		
		// yyyy-MM-ddTHH:mm:ss.MMMZ
		// 012345678901234567890123
		
		year = s.substring(0, 4);
		month = s.substring(5, 7);
		day = s.substring(8, 10);
		hour = s.substring(11, 13);
		minute = s.substring(14, 16);
		seconds = s.substring(17, 19);
		
		calendar = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
		calendar.set(Calendar.YEAR, Integer.parseInt(year));
		calendar.set(Calendar.MONTH, Integer.parseInt(month) - 1);
		calendar.set(Calendar.DAY_OF_MONTH, Integer.parseInt(day));
		calendar.set(Calendar.HOUR_OF_DAY, Integer.parseInt(hour));
		calendar.set(Calendar.MINUTE, Integer.parseInt(minute));
		calendar.set(Calendar.SECOND, Integer.parseInt(seconds));
		try {
			// millis may not exist so catch the exception
			millis = s.substring(20);
			
			// remove the trailing 'Z' from the millis
			if (millis.endsWith("Z")) {
				millis = millis.substring(0, millis.length()-1);
			}
			
			calendar.set(Calendar.MILLISECOND, Integer.parseInt(millis));
		} catch (Exception e) {
			calendar.set(Calendar.MILLISECOND, 0);
		}
		
		return calendar.getTime();
	}
	
	/**
	 * Converts the given string (as in '2008-10-03') using the given
	 * pattern (as in 'yyyy-MM-dd') to a Date; if adjust parameter is 
	 * set to true, add to the result so that the result represents 
	 * the last milli at the end of the date
	 * @param date
	 * @param pattern
	 * @param adjust
	 * @return DTS equivalent of the given date using the given pattern
	 */
	public Date convertToDate(String date, String pattern, boolean adjust) {
		try {
			SimpleDateFormat sdf = new SimpleDateFormat(pattern);
			Date d = sdf.parse(date);
			
			Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
			calendar.setTime(d);
			calendar.set(Calendar.HOUR_OF_DAY, 0);
			calendar.set(Calendar.MINUTE, 0);
			calendar.set(Calendar.SECOND, 0);
			calendar.set(Calendar.MILLISECOND, 0);
			
			if (adjust) {
				calendar.add(Calendar.DATE, 1);
			}
			
			return calendar.getTime();
		} catch (Exception e) {
			return null;
		}
	}
	
	/**
	 * Converts the given java.util.Date value as an ISO 8601 date string.
	 * 
	 * <P>ISO 8601 date strings have the form <pre>yyyy-MM-ddTHH:mm:ss.MMMZ</pre>
	 * 
	 * @param d The java.util.Date value to format
	 * @return String
	 * @throws SensorEventException
	 */
	public String formatISO8601Date(Date d) {
		if (d == null) {
			return null;
		}
		
		return formatISO8601Date(d, TimeZone.getTimeZone("GMT"));
	}
	
	/**
	 * Converts the date to an ISO 8601 date string for the specified timezone
	 * @param date
	 * @param timeZone
	 * @return
	 */
	public String formatISO8601Date(Date date, TimeZone timeZone) {
		// yyyy-MM-ddTHH:mm:ss.MMMZ
		
		Calendar calendar = null;
		if (timeZone == null) {
			calendar = Calendar.getInstance();
		} else {
			calendar = Calendar.getInstance(timeZone);
		}
		
		calendar.setTime(date);
		StringBuffer buffer = new StringBuffer(29);
		this.appendDate(calendar, buffer);
		buffer.append("T");
		this.appendTime(calendar, buffer);
		this.appendTimeZone(calendar, buffer);
		return buffer.toString();
	}
	
	private void appendDate(Calendar calendar, StringBuffer buffer) {
		int year = calendar.get(Calendar.YEAR);
        buffer.append(year);
		buffer.append('-');
		int month = calendar.get(Calendar.MONTH) + 1;
		if (month < 10) buffer.append('0');
		buffer.append(month);
		buffer.append('-');
		int day = calendar.get(Calendar.DAY_OF_MONTH);
		if (day < 10) buffer.append('0');        
		buffer.append(day);
	}
	
	private void appendTime(Calendar calendar, StringBuffer buffer) {
		int hour = calendar.get(Calendar.HOUR_OF_DAY);
		if (hour < 10) buffer.append('0');            
		buffer.append(hour);
		buffer.append(':');
		int minute = calendar.get(Calendar.MINUTE);
		if (minute < 10) buffer.append('0');
		buffer.append(minute);      
		buffer.append(':');
		int second = calendar.get(Calendar.SECOND);
		if (second < 10) buffer.append('0'); 
		buffer.append(second);
		buffer.append('.');
		int millisecond = calendar.get(Calendar.MILLISECOND);
		if (millisecond < 100) buffer.append('0');
		if (millisecond < 10) buffer.append('0');    
		buffer.append(millisecond);     
	}
	
	private void appendTimeZone(Calendar calendar, StringBuffer buffer) {
		long offset = calendar.getTimeZone().getRawOffset();
		long hour = (offset < 0 ? -offset : offset) / 3600000;
		long minute = ((offset < 0 ? -offset : offset) % 3600000) / 60000;
		if (hour == 0 && minute == 0) {
			buffer.append("Z");
			return;
		}
		buffer.append(offset < 0 ? '-' : '+');
		if (hour < 10) buffer.append('0');
		buffer.append(hour);
		buffer.append(':');
		if (minute < 10) buffer.append('0');
		buffer.append(minute);
	}
	
}
