/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.ejb;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.ejb.CreateException;
import javax.ejb.SessionContext;

import com.ibm.premises.ct.dto.PremisesLocation;
import com.ibm.premises.ct.dto.PremisesPrinter;
import com.ibm.premises.ct.dto.PremisesProfile;
import com.ibm.premises.ct.exceptions.PremisesFacadeException;
import com.ibm.premises.ct.utils.Utils;
import com.ibm.rfid.epcg.tds.EPC;
import com.ibm.rfid.epcg.tds.EPCFactory;
import com.ibm.rfid.premises.app.access.PremisesAccess;
import com.ibm.rfid.premises.app.access.PremisesContext;
import com.ibm.rfid.premises.app.entities.supplychain.PackType;
import com.ibm.rfid.premises.app.entities.supplychain.SupplyChainProfile;
import com.ibm.rfid.premises.entities.Device;
import com.ibm.rfid.premises.entities.Location;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Container Tracking Premises Server API facade.
 */
public class PremisesSessionFacadeBean implements javax.ejb.SessionBean {
	private static final long serialVersionUID = 1L;
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
		+ "All Rights Reserved * Licensed Materials - Property of IBM "
		+ "US Government Users Restricted Rights - Use, duplication or "
		+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	public static final String CATEGORY_PRINTER = "Printer";
	public static final String CATEGORY_LOGICAL_PRINTER = "Logical Printer";
	public static final String CATEGORY_READER = "Reader";
	
	public static final String LOCATION_STOPPED = "OFF";
	public static final String LOCATION_STARTED = "ON";
	public static final String LOCATION_STOPPED_60 = "stop_reading";
	public static final String LOCATION_STARTED_60 = "start_reading";
	
	private static Utils utils;
	private static ResourceBundle rb;
	private SessionContext mySessionCtx;
	
	/**
	 * Returns all locations currently defined in the Premises Server
	 * @return the locations
	 * @throws PremisesFacadeException
	 */
	@SuppressWarnings("unchecked")
	public SortedSet<PremisesLocation> getAllLocations() throws PremisesFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "getAllLocations");
			
			PremisesAccess access = PremisesContext.getPremisesAccess();
			Location[] allPremisesLocations = access.getAllLocation();
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.premises.trace.info.location.count"), allPremisesLocations.length);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getAllLocations", message);
			}
			
			SortedSet<PremisesLocation> locations = new TreeSet<PremisesLocation>();
			for (int i=0; i<allPremisesLocations.length; i++) {
				if (!allPremisesLocations[i].isContainerLocation()) {
					// only process leaf locations
					String id = allPremisesLocations[i].getLocationId();
					String description = allPremisesLocations[i].getUserName();
					boolean started = this.isLocationStarted(id);
					Map metaData = allPremisesLocations[i].getMetadata();
					
					PremisesLocation premisesLocation = new PremisesLocation(id);
					premisesLocation.setDescription(description);
					premisesLocation.setStarted(started);
					premisesLocation.setMetaData(metaData);
					
					locations.add(premisesLocation);
					
					if (logger.isTraceEnabled()) {
						String message = utils.formatMessage(rb.getString("ibmct.premises.trace.info.location"), 
								new Object[] {id, description, started, metaData.toString()});
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getAllLocations", message);
					}
				}
			}
			
			return locations;
		} catch(Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "getAllLocations", e);
			e.printStackTrace();
			throw new PremisesFacadeException(e);
		} finally {
			logger.traceExit(this, "getAllLocations");
		}
	}
	
	/**
	 * Returns the Premises location
	 * @param locationId
	 * @return the location
	 * @throws PremisesFacadeException
	 */
	@SuppressWarnings("unchecked")
	public PremisesLocation getLocation(String locationId) throws PremisesFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "getLocation");
			
			PremisesAccess access = PremisesContext.getPremisesAccess();
			Location premisesLocation = access.getLocation(locationId);

			String description = premisesLocation.getUserName();
			boolean started = this.isLocationStarted(locationId);
			Map metaData = premisesLocation.getMetadata();
					
			PremisesLocation location = new PremisesLocation(locationId);
			location.setDescription(description);
			location.setStarted(started);
			location.setMetaData(metaData);
					
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.premises.trace.info.location"), 
						new Object[] {locationId, description, started, metaData.toString()});
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getLocation", message);
			}
			
			return location;
		} catch(Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "getLocation", e);
			e.printStackTrace();
			throw new PremisesFacadeException(e);
		} finally {
			logger.traceExit(this, "getLocation");
		}
	}
	
	/**
	 * Returns true if the location is started
	 * @param locationId
	 * @return
	 * @throws PremisesFacadeException
	 */
	public boolean isLocationStarted(String locationId) throws PremisesFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "isLocationStarted");
			
			String status = this.getLocationStatus(locationId);
			
			boolean started = false;
			if (LOCATION_STARTED.equals(status) || LOCATION_STARTED_60.equals(status)) {
				started = true;
			}
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.premises.trace.location.isstarted"), locationId, started);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "isLocationStarted", message);
			}
			
			return started;
		} catch(Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "isLocationStarted", e);
			e.printStackTrace();
			throw new PremisesFacadeException(e);
		} finally {
			logger.traceExit(this, "isLocationStarted");
		}
	}
	
	/**
	 * Returns true if the location is stopped
	 * @param locationId
	 * @return
	 * @throws PremisesFacadeException
	 */
	public boolean isLocationStopped(String locationId) throws PremisesFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "isLocationStopped");
			
			String status = this.getLocationStatus(locationId);
			
			boolean stopped = false;
			if (LOCATION_STOPPED.equals(status) || LOCATION_STOPPED_60.equals(status)) {
				stopped = true;
			}
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.premises.trace.location.isstopped"), locationId, stopped);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "isLocationStopped", message);
			}
			
			return stopped;
		} catch(Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "isLocationStopped", e);
			e.printStackTrace();
			throw new PremisesFacadeException(e);
		} finally {
			logger.traceExit(this, "isLocationStopped");
		}
	}
	
	/**
	 * Returns the location's status
	 * @param locationId
	 * @return the location status
	 * @throws PremisesFacadeException
	 */
	public String getLocationStatus(String locationId) throws PremisesFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "getLocationStatus");
			
			PremisesAccess access = PremisesContext.getPremisesAccess();
			String status = access.getDeviceStatus(locationId);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.premises.trace.info.location.status"), locationId, status);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getLocationStatus", message);
			}
			
			return status;
		} catch(Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "getLocationStatus", e);
			e.printStackTrace();
			throw new PremisesFacadeException(e);
		} finally {
			logger.traceExit(this, "getLocationStatus");
		}
	}
	
	/**
	 * Starts the location using the process and supplier
	 * @param locationId
	 * @param process
	 * @param supplierId
	 * @throws PremisesFacadeException
	 */
	public void startLocation(String locationId, String process, String supplierId) throws PremisesFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "startLocation");
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.premises.trace.info.start.location"), locationId, process, supplierId);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "startLocation", message);
			}
			
			Map<String, String> map = new HashMap<String, String>();
			map.put(PremisesSessionFacadeConstants.EPCIS_EVENT_TIMEZONE_OFFSET, PremisesSessionFacadeConstants.EPCIS_EVENT_TIMEZONE_OFFSET_GMT);
			map.put(PremisesSessionFacadeConstants.EPCIS_BIZ_STEP_KEY, this.toEpcisBizStep(process));
			map.put(PremisesSessionFacadeConstants.EPCIS_READ_POINT_KEY, this.toEpcisReadPoint(locationId));
			map.put(PremisesSessionFacadeConstants.EPCIS_DISPOSITION_KEY, this.toEpcisDisposition(process));
			
			String bizLocationId = this.getPremisesProperty(PremisesSessionFacadeConstants.PROP_BIZ_LOCATION);
			map.put(PremisesSessionFacadeConstants.EPCIS_BIZ_LOCATION_KEY, this.toEpcisBizLocation(bizLocationId));
			
			if (PremisesSessionFacadeConstants.PROCESS_SHIPPING.equalsIgnoreCase(process)) {
				map.put(PremisesSessionFacadeConstants.EPCIS_SHIP_TO_BIZ_LOCATION_KEY, this.toEpcisShipToBizLocation(supplierId));
			} else if (PremisesSessionFacadeConstants.PROCESS_RECEIVING.equalsIgnoreCase(process)) {
				map.put(PremisesSessionFacadeConstants.EPCIS_SHIP_FROM_BIZ_LOCATION_KEY, this.toEpcisShipFromBizLocation(supplierId));
			} else {
				// do nothing
			}
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.premises.trace.info.location.metadata"), locationId, map.toString());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "startLocation", message);
			}

			PremisesAccess access = PremisesContext.getPremisesAccess();
			
			// delete the old metaData from the database
			access.deleteLocationMetaData(locationId, PremisesSessionFacadeConstants.EPCIS_EVENT_TIMEZONE_OFFSET);
			access.deleteLocationMetaData(locationId, PremisesSessionFacadeConstants.EPCIS_BIZ_STEP_KEY);
			access.deleteLocationMetaData(locationId, PremisesSessionFacadeConstants.EPCIS_BIZ_LOCATION_KEY);
			access.deleteLocationMetaData(locationId, PremisesSessionFacadeConstants.EPCIS_READ_POINT_KEY);
			access.deleteLocationMetaData(locationId, PremisesSessionFacadeConstants.EPCIS_DISPOSITION_KEY);
			access.deleteLocationMetaData(locationId, PremisesSessionFacadeConstants.EPCIS_SHIP_TO_BIZ_LOCATION_KEY);
			access.deleteLocationMetaData(locationId, PremisesSessionFacadeConstants.EPCIS_SHIP_FROM_BIZ_LOCATION_KEY);
			
			// delete the old metaData on data capture
			access.sendLocationMetaData(locationId, true, null);
			
			// save the new metaData to the database
			access.setLocationMetaData(locationId, (HashMap) map);
			
			// save the new metaData on data capture
			access.sendLocationMetaData(locationId, false, map);
			
			// turn on the location
			access.startLocation(locationId);
		} catch(Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "startLocation", e);
			e.printStackTrace();
			throw new PremisesFacadeException(e);
		} finally {
			logger.traceExit(this, "startLocation");
		}
	}
	
	/**
	 * Returns the EPCIS meta-data for the process
	 * @param process
	 * @return
	 */
	public String toEpcisBizStep(String process) {
		return PremisesSessionFacadeConstants.EPCIS_BIZ_STEP_PREFIX + process;
	}
	
	/**
	 * Returns the EPCIS meta-data for the process
	 * @param process
	 * @return
	 */
	public String toEpcisDisposition(String process) {
		if (PremisesSessionFacadeConstants.PROCESS_SHIPPING.equalsIgnoreCase(process)) {
			return PremisesSessionFacadeConstants.EPCIS_DISPOSITION_PREFIX + PremisesSessionFacadeConstants.DISPOSITION_IN_TRANSIT;
		} else if (PremisesSessionFacadeConstants.PROCESS_RECEIVING.equalsIgnoreCase(process)) {
			return PremisesSessionFacadeConstants.EPCIS_DISPOSITION_PREFIX + PremisesSessionFacadeConstants.DISPOSITION_IN_PROGRESS;
		} else {
			return PremisesSessionFacadeConstants.EPCIS_DISPOSITION_PREFIX + PremisesSessionFacadeConstants.DISPOSITION_UNKNOWN;
		}
	}
	
	/**
	 * Extracts the process from the EPCIS meta-data
	 * @param bizStep
	 * @return
	 */
	public String extractProcessFromEpcisBizStep(String bizStep) {
		if (bizStep == null) {
			return "";
		}
		
		if (!bizStep.startsWith(PremisesSessionFacadeConstants.EPCIS_BIZ_STEP_PREFIX)) {
			return "";
		}
		
		return bizStep.substring(PremisesSessionFacadeConstants.EPCIS_BIZ_STEP_PREFIX.length());
	}
	
	/**
	 * Extracts the disposition from the EPCIS meta-data
	 * @param disposition
	 * @return
	 */
	public String extractDispositionFromEpcisDisposition(String disposition) {
		if (disposition == null) {
			return "";
		}
		
		if (!disposition.startsWith(PremisesSessionFacadeConstants.EPCIS_DISPOSITION_PREFIX)) {
			return "";
		}
		
		return disposition.substring(PremisesSessionFacadeConstants.EPCIS_DISPOSITION_PREFIX.length());
	}
	
	/**
	 * Returns the EPCIS meta-data for the locationId
	 * @param locationId
	 * @return
	 */
	public String toEpcisBizLocation(String locationId) {
		return PremisesSessionFacadeConstants.EPCIS_BIZ_LOCATION_PREFIX + locationId;
	}
	
	/**
	 * Returns the EPCIS meta-data for the readPoint
	 * @param locationId
	 * @return
	 */
	public String toEpcisReadPoint(String locationId) {
		return PremisesSessionFacadeConstants.EPCIS_READ_POINT_PREFIX + locationId;
	}
	
	/**
	 * Extracts the location ID from the EPCIS meta-data
	 * @param bizStep
	 * @return
	 */
	public String extractLocationFromEpcisBizLocation(String bizLocation) {
		if (bizLocation == null) {
			return "";
		}
		
		if (!bizLocation.startsWith(PremisesSessionFacadeConstants.EPCIS_BIZ_LOCATION_PREFIX)) {
			return "";
		}
		
		return bizLocation.substring(PremisesSessionFacadeConstants.EPCIS_BIZ_LOCATION_PREFIX.length());
	}
	
	/**
	 * Extracts the location ID from the EPCIS meta-data
	 * @param readPoint
	 * @return
	 */
	public String extractLocationFromEpcisReadLocation(String readPoint) {
		if (readPoint == null) {
			return "";
		}
		
		if (!readPoint.startsWith(PremisesSessionFacadeConstants.EPCIS_READ_POINT_PREFIX)) {
			return "";
		}
		
		return readPoint.substring(PremisesSessionFacadeConstants.EPCIS_READ_POINT_PREFIX.length());
	}
	
	/**
	 * Returns the EPCIS meta-data for the supplierId
	 * @param supplierId
	 * @return
	 */
	public String toEpcisShipToBizLocation(String supplierId) {
		return PremisesSessionFacadeConstants.EPCIS_SHIP_TO_BIZ_LOCATION_PREFIX + supplierId;
	}
	
	/**
	 * Extracts the supplier ID from the EPCIS meta-data
	 * @param shipToBizLocation
	 * @return
	 */
	public String extractSupplierFromEpcisShipToBizLocation(String shipToBizLocation) {
		if (shipToBizLocation == null) {
			return "";
		}
		
		if (!shipToBizLocation.startsWith(PremisesSessionFacadeConstants.EPCIS_SHIP_TO_BIZ_LOCATION_PREFIX)) {
			return "";
		}
		
		return shipToBizLocation.substring(PremisesSessionFacadeConstants.EPCIS_SHIP_TO_BIZ_LOCATION_PREFIX.length());
	}
	
	/**
	 * Returns the EPCIS meta-data for the supplierId
	 * @param supplierId
	 * @return
	 */
	public String toEpcisShipFromBizLocation(String supplierId) {
		return PremisesSessionFacadeConstants.EPCIS_SHIP_FROM_BIZ_LOCATION_PREFIX + supplierId;
	}
	
	/**
	 * Extracts the supplier ID from the EPCIS meta-data
	 * @param shipFromBizLocation
	 * @return
	 */
	public String extractSupplierFromEpcisShipFromBizLocation(String shipFromBizLocation) {
		if (shipFromBizLocation == null) {
			return "";
		}
		
		if (!shipFromBizLocation.startsWith(PremisesSessionFacadeConstants.EPCIS_SHIP_FROM_BIZ_LOCATION_PREFIX)) {
			return "";
		}
		
		return shipFromBizLocation.substring(PremisesSessionFacadeConstants.EPCIS_SHIP_FROM_BIZ_LOCATION_PREFIX.length());
	}
	
	/**
	 * Stops the location
	 * @param locationId
	 * @throws PremisesFacadeException
	 */
	public void stopLocation(String locationId) throws PremisesFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "stopLocation");
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.premises.trace.info.stop.location"), locationId);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "stopLocation", message);
			}

			PremisesAccess access = PremisesContext.getPremisesAccess();
			
			// to avoid timing issues with events that are not yet processed, do not delete the
			// old metadata when stopping the portal
			
//			// delete the old metaData
//			access.deleteLocationMetaData(locationId, PremisesSessionFacadeConstants.EPCIS_EVENT_TIMEZONE_OFFSET);
//			access.deleteLocationMetaData(locationId, PremisesSessionFacadeConstants.EPCIS_BIZ_STEP_KEY);
//			access.deleteLocationMetaData(locationId, PremisesSessionFacadeConstants.EPCIS_BIZ_LOCATION_KEY);
//			access.deleteLocationMetaData(locationId, PremisesSessionFacadeConstants.EPCIS_READ_POINT_KEY);
//			access.deleteLocationMetaData(locationId, PremisesSessionFacadeConstants.EPCIS_DISPOSITION_KEY);
//			access.deleteLocationMetaData(locationId, PremisesSessionFacadeConstants.EPCIS_SHIP_TO_BIZ_LOCATION_KEY);
//			access.deleteLocationMetaData(locationId, PremisesSessionFacadeConstants.EPCIS_SHIP_FROM_BIZ_LOCATION_KEY);
			
//			// delete the old metaData on data capture
//			access.sendLocationMetaData(locationId, true, null);
			
			// turn off the location
			access.stopLocation(locationId);
		} catch(Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "stopLocation", e);
			e.printStackTrace();
			throw new PremisesFacadeException(e);
		} finally {
			logger.traceExit(this, "stopLocation");
		}
	}
	
	/**
	 * Returns the EPC profiles currently defined in the Premises Server with the given ID
	 * @return the profile
	 * @throws PremisesFacadeException
	 */
	@SuppressWarnings("unchecked")
	public PremisesProfile getProfile(String id) throws PremisesFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "getProfile");
			
			PremisesAccess access = PremisesContext.getPremisesAccess();
			SupplyChainProfile premisesProfile = access.getSupplyChainProfile(id);
			if (premisesProfile == null) {
				String message = utils.formatMessage(rb.getString("ibmct.premises.trace.error.profile.not.found"), id);
				if (logger.isTraceEnabled()) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getProfile", message);
				}

				throw new PremisesFacadeException(message);
			}

			String description = premisesProfile.getDescription();
			
			List<String> packTypeNames = new ArrayList<String>();
			for (Iterator<PackType> i=premisesProfile.getPackTypes().values().iterator(); i.hasNext(); ) {
				PackType packType = i.next();
				packTypeNames.add(packType.getName());
			}
				
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.premises.trace.info.profile"), id, description);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getProfile", message);
			}

			PremisesProfile profile = new PremisesProfile(id, description);
			profile.setPackTypeNames(packTypeNames);
			
			return profile;
		} catch(Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "getProfile", e);
			e.printStackTrace();
			throw new PremisesFacadeException(e);
		} finally {
			logger.traceExit(this, "getProfile");
		}
	}
	
	/**
	 * Returns all EPC profiles currently defined in the Premises Server
	 * @return the profiles
	 * @throws PremisesFacadeException
	 */
	@SuppressWarnings("unchecked")
	public SortedSet<PremisesProfile> getAllProfiles() throws PremisesFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "getAllProfiles");
			
			PremisesAccess access = PremisesContext.getPremisesAccess();
			SupplyChainProfile[] allPremisesProfiles = access.getSupplyChainProfiles();
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.premises.trace.info.profile.count"), allPremisesProfiles.length);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getAllProfiles", message);
			}
			
			SortedSet<PremisesProfile> profiles = new TreeSet<PremisesProfile>();
			for (int i=0; i<allPremisesProfiles.length; i++) {
				String id = allPremisesProfiles[i].getProfileId();
				String description = allPremisesProfiles[i].getDescription();
				
				List<String> packTypeNames = new ArrayList<String>();
				for (Iterator<PackType> j=allPremisesProfiles[i].getPackTypes().values().iterator(); j.hasNext(); ) {
					PackType packType = j.next();
					packTypeNames.add(packType.getName());
				}
				
				if (logger.isTraceEnabled()) {
					String message = utils.formatMessage(rb.getString("ibmct.premises.trace.info.profile"), id, description);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getAllProfiles", message);
				}
				
				PremisesProfile profile = new PremisesProfile(id, description);
				profile.setPackTypeNames(packTypeNames);
				
				profiles.add(profile);
			}

			return profiles;
		} catch(Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "getAllProfiles", e);
			e.printStackTrace();
			throw new PremisesFacadeException(e);
		} finally {
			logger.traceExit(this, "getAllProfiles");
		}
	}
	
	/**
	 * Returns all printers currently defined in the Premises Server
	 * @return the printers
	 * @throws PremisesFacadeException
	 */
	public SortedSet<PremisesPrinter> getAllPrinters() throws PremisesFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "getAllPrinters");
			
			PremisesAccess access = PremisesContext.getPremisesAccess();
			Device[] devices = access.getAllDevices();
			
			SortedSet<PremisesPrinter> printers = new TreeSet<PremisesPrinter>();
			for (int i=0; i<devices.length; i++) {
				Device device = devices[i];
				if (CATEGORY_PRINTER.equals(device.getCategory()) || CATEGORY_LOGICAL_PRINTER.equals(device.getCategory())) {
					String id = device.getDeviceId();
					String description = device.getUserName();
					
					if (logger.isTraceEnabled()) {
						String message = utils.formatMessage(rb.getString("ibmct.premises.trace.info.printer"), id, description);
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getAllPrinters", message);
					}
					
					printers.add(new PremisesPrinter(id, description));
				}
			}
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.premises.trace.info.printer.count"), printers.size());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getAllPrinters", message);
			}

			return printers;
		} catch(Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "getAllPrinters", e);
			e.printStackTrace();
			throw new PremisesFacadeException(e);
		} finally {
			logger.traceExit(this, "getAllPrinters");
		}
	}
	
	/**
	 * Returns a printer currently defined in the Premises Server
	 * @param id the printer ID to return
	 * @return the printers
	 * @throws PremisesFacadeException
	 */
	public PremisesPrinter getPrinter(String id) throws PremisesFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "getPrinter");
			
			PremisesAccess access = PremisesContext.getPremisesAccess();
			Device device = access.getDevice(id);
			if (device == null) {
				String message = utils.formatMessage(rb.getString("ibmct.premises.trace.error.printer.not.found"), id);
				if (logger.isTraceEnabled()) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getPrinter", message);
				}

				throw new PremisesFacadeException(message);
			}
		
			String description = device.getUserName();

			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.premises.trace.info.printer"), id, description);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getPrinter", message);
			}
			
			PremisesPrinter printer = new PremisesPrinter(id, description);

			return printer;
		} catch(Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "getPrinter", e);
			e.printStackTrace();
			throw new PremisesFacadeException(e);
		} finally {
			logger.traceExit(this, "getPrinter");
		}
	}
	
	/**
	 * Returns the value of the specified Premises property key; returns null if the key
	 * is not defined
	 * @param key
	 * @return the value
	 * @throws PremisesFacadeException
	 */
	public String getPremisesProperty(String key) throws PremisesFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "getPremisesProperty");
			
			PremisesAccess access = PremisesContext.getPremisesAccess();
			String value = access.getPremisesPropertiesKeyValue(key);
			if (value != null && value.length() == 0) {
				value = null;
			}
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.premises.trace.info.property.lookup"), key, value);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getPremisesProperty", message);
			}
			
			return value;
		} catch(Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "getPremisesProperty", e);
			e.printStackTrace();
			throw new PremisesFacadeException(e);
		} finally {
			logger.traceExit(this, "getPremisesProperty");
		}
	}
	
	/**
	 * Returns the value of the specified Premises property key; if the key is not defined, returns the
	 * specified defaultValue
	 * @param key
	 * @param defaultValue
	 * @return the value
	 * @throws PremisesFacadeException
	 */
	public String getPremisesProperty(String key, String defaultValue) throws PremisesFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "getPremisesPropertyWithDefault");
			
			String value = this.getPremisesProperty(key);
			if (value != null && value.length() > 0) {
				return value;
			} else {
				return defaultValue;
			}
		} finally {
			logger.traceExit(this, "getPremisesPropertyWithDefault");
		}
	}
	
	/**
	 * Return the epcUrn converted to its raw URI value
	 * @param epcUrn
	 * @return
	 */
	public String toEpcRawURI(String epcUrn) throws PremisesFacadeException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "toEpcRawURI");

			EPC epc = EPCFactory.encodeStandard(epcUrn);
			String raw = epc.toHexadecimal();
			return raw;
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "toEpcRawURI", e);
			e.printStackTrace();
			return null;	// do not throw an exception in case the epcUrn is not a valid URN
		} finally {
			logger.traceExit(this, "toEpcRawURI");
		}
		
	}
	
	/**
	 * getSessionContext
	 */
	public javax.ejb.SessionContext getSessionContext() {
		return mySessionCtx;
	}
	
	/**
	 * setSessionContext
	 */
	public void setSessionContext(javax.ejb.SessionContext ctx) {
		mySessionCtx = ctx;
	}
	
	/**
	 * ejbCreate
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
		utils = Utils.singleton();
		Logger logger = utils.logger;
		rb = ResourceBundle.getBundle("com.ibm.premises.ct.ejb.resources.PremisesMessages");

		try {
			logger.traceEntry(this, "ejbCreate");
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "ejbCreate", e);
			e.printStackTrace();
			throw new CreateException(e.getMessage());
		} finally {
			logger.traceExit(this, "ejbCreate");
		}
	}
	
	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	
	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}
	
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}
}
