/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.ejb;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;

import com.ibm.premises.ct.dto.Asset;
import com.ibm.premises.ct.dto.PremisesLocation;
import com.ibm.premises.ct.ejb.RucSessionFacadeConstants.ReportingRuc;
import com.ibm.premises.ct.ejb.RucSessionFacadeConstants.ValidationRuc;
import com.ibm.premises.ct.ejb.exceptions.TaskAgentException;
import com.ibm.premises.ct.ejb.exceptions.TaskAgentRuntimeException;
import com.ibm.premises.ct.utils.Utils;
import com.ibm.rfid.premises.logger.Logger;
import com.ibm.sensorevent.model.IPayload;
import com.ibm.sensorevent.model.IPayloadMetaData;
import com.ibm.sensorevent.model.ISensorEvent;
import com.ibm.sensorevent.model.generic.IGenericGroup;
import com.ibm.sensorevent.model.payload.EPCTag;
import com.ibm.sensorevent.model.payload.PassiveRFIDAggregatedTagReadPayload;
import com.ibm.sensorevent.model.payload.PassiveRFIDTagRead;
import com.ibm.sensorevent.model.payload.PassiveRFIDTagReadPayload;

/**
 * Bean implementation class for Enterprise Bean: TagReadTaskAgent
 */
public class TagReadTaskAgentBean extends com.ibm.sensorevent.engine.baseagent.IBMSEAbstractTaskAgent
implements javax.ejb.MessageDrivenBean, javax.jms.MessageListener {

	private static final long serialVersionUID = 1L;
	private javax.ejb.MessageDrivenContext fMessageDrivenCtx;
	private static Utils utils;
	private static ResourceBundle rb;
	protected static RucSessionFacadeLocalHome rucHome;
	protected static JdbcSessionFacadeLocalHome jdbcHome;
	protected static PremisesSessionFacadeLocalHome premisesHome;
	
	/* (non-Javadoc)
	 * @see com.ibm.sensorevent.engine.baseagent.IBMSEAbstractTaskAgent#onIBMSensorEvent(com.ibm.sensorevent.model.ISensorEvent)
	 */
	@Override
	protected void onIBMSensorEvent(ISensorEvent event) {
		Logger logger = utils.logger;
		
		try {
			RucSessionFacadeLocal ruc = rucHome.create();
			
			logger.traceEntry(this, "onIBMSensorEvent");
		
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "onIBMSensorEvent", "event=" + event);
			}
			
			// collect the tag reads from the event
			Collection<EPCTag> epcTagReads = this.collectTagReads(event);
			
			// are the tags commissioned?
			Map<String, String> assetCache = new HashMap<String, String>();
			for (Iterator<EPCTag> i=epcTagReads.iterator(); i.hasNext(); ) {
				EPCTag epcTag = i.next();
				String tagUrn = epcTag.getAsTagURI();
				
				if (logger.isTraceEnabled()) {
					String message = utils.formatMessage(rb.getString("ibmct.tagreadagent.validating.tag"), tagUrn);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "onIBMSensorEvent", message);
				}
				
				boolean isValid = ruc.validate(tagUrn);
				
				if (isValid) {
					// tag is already commissioned
					if (logger.isTraceEnabled()) {
						String message = utils.formatMessage(rb.getString("ibmct.tagreadagent.validate.is.commissioned"), tagUrn);
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "onIBMSensorEvent", message);
					}
				} else {
					// commission the tag
					if (logger.isTraceEnabled()) {
						String message = utils.formatMessage(rb.getString("ibmct.tagreadagent.validate.is.not.commissioned"), tagUrn);
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "onIBMSensorEvent", message);
					}
					
					// query the asset's type
					int epcAssetType = this.extractEpcAssetType(tagUrn);
					String assetType = null;
					if (epcAssetType < 0) {
						assetType = "";
					} else {
						assetType = String.valueOf(epcAssetType);
					}
										
					ruc.commission(event.getHeader().getSourceId(), tagUrn, assetType);
				}
				
			}
			
			// put the tags extensions into the event payload metadata
			event = this.addExtensions(assetCache, event, epcTagReads);
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "onIBMSensorEvent", "event=" + event);
			}
			
			// put the location user name into the event payload metadata
			event = this.addLocationUserName(event);
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "onIBMSensorEvent", "event=" + event);
			}
			
			// put the shipToBusinessLocation/shipFromBusinessLocation into the event payload metadata
			event = this.addShipToFromBusinessLocation(event);
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "onIBMSensorEvent", "event=" + event);
			}
			
			// send the observe event
			ruc.observe(event);
			
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "onIBMSensorEvent", e);
			e.printStackTrace();
			throw new TaskAgentRuntimeException(e);
		} finally {
			logger.traceExit(this, "onIBMSensorEvent");
		}
		
	}
	
	/**
	 * Add the location user name to the event's payload metadata
	 * @param event
	 * @return
	 * @throws TaskAgentException
	 */
	private ISensorEvent addLocationUserName(ISensorEvent event) throws TaskAgentException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "addLocationName");
			
			// the event's sourceId is the locationId
			String sourceId = event.getHeader().getSourceId();
			
			// find the location's user name
			String locationUserName = sourceId;			
			try {
				PremisesSessionFacadeLocal premises = premisesHome.create();
				PremisesLocation premisesLocation = premises.getLocation(sourceId);
				if (premisesLocation != null) {
					locationUserName = premisesLocation.getDescription();
				}
			} catch (Exception e) {
				locationUserName = sourceId;
			}
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.tagreadagent.location.username"), sourceId, locationUserName);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "addLocationName", message);
			}
			
			// add the location's user name to the event's payload metadata
			IPayloadMetaData pmd = event.getPayloadMetaData();
			pmd.addStringAttribute(ReportingRuc.EPCIS_EXT + "." + ReportingRuc.EXT_LOCATION_USER_NAME, locationUserName);
			
			return event;			
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "addLocationName", e);
			e.printStackTrace();
			throw new TaskAgentRuntimeException(e);
		} finally {
			logger.traceExit(this, "addLocationName");
		}
	}
	
	/**
	 * Add the shipToBusinessLocation/shipFromBusinessLocation to the event's payload metadata
	 * @param event
	 * @return
	 * @throws TaskAgentException
	 */
	private ISensorEvent addShipToFromBusinessLocation(ISensorEvent event) throws TaskAgentException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "addShipToFromBusinessLocation");
			
			// the event's sourceId is the locationId
			String sourceId = event.getHeader().getSourceId();
			
			// update the payload metadata with either shipToBusinessLocation or shipFromBusinessLocation
			try {
				IPayloadMetaData pmd = event.getPayloadMetaData();
				PremisesSessionFacadeLocal premises = premisesHome.create();
				PremisesLocation premisesLocation = premises.getLocation(sourceId);
				if (premisesLocation != null) {
					Map<String, String> metaData = premisesLocation.getMetaData();
					if (metaData.containsKey(PremisesSessionFacadeConstants.EPCIS_SHIP_TO_BIZ_LOCATION_KEY)) {
						String shipToBusinessLocation = metaData.get(PremisesSessionFacadeConstants.EPCIS_SHIP_TO_BIZ_LOCATION_KEY);
						pmd.addStringAttribute("epcis." + PremisesSessionFacadeConstants.EPCIS_SHIP_TO_BIZ_LOCATION_KEY, shipToBusinessLocation);
					} else if (metaData.containsKey(PremisesSessionFacadeConstants.EPCIS_SHIP_FROM_BIZ_LOCATION_KEY)) {
						String shipFromBusinessLocation = metaData.get(PremisesSessionFacadeConstants.EPCIS_SHIP_FROM_BIZ_LOCATION_KEY);
						pmd.addStringAttribute("epcis." + PremisesSessionFacadeConstants.EPCIS_SHIP_FROM_BIZ_LOCATION_KEY, shipFromBusinessLocation);
					}
				}
			} catch (Exception e) {
				// no-op
			}
			
			return event;			
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "addShipToFromBusinessLocation", e);
			e.printStackTrace();
			throw new TaskAgentRuntimeException(e);
		} finally {
			logger.traceExit(this, "addShipToFromBusinessLocation");
		}
	}
	
	/**
	 * Gather all EPCTags from the event
	 * @param event
	 * @return
	 * @throws TaskAgentException
	 */
	private Collection<EPCTag> collectTagReads(ISensorEvent event) throws TaskAgentException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "collectTagReads");
			
			Collection<EPCTag> epcTagReads = new ArrayList<EPCTag>();
			
			IPayload payload = event.getPayload();
	
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.tagreadagent.payload.type"), payload.getClass().getName());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "collectTagReads", message);
			}
			
			if (payload instanceof PassiveRFIDAggregatedTagReadPayload) {
				// aggregated tag read
				PassiveRFIDTagRead tagRead = null;
				EPCTag epcTag = null;
				for (Iterator i=((PassiveRFIDAggregatedTagReadPayload) payload).getTagReads().iterator(); i.hasNext(); ) {
					tagRead = (PassiveRFIDTagRead) i.next();
					epcTag = (EPCTag) tagRead.getTag();
					
					if (logger.isTraceEnabled()) {
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "collectTagReads", "epcTag=" + epcTag);
					}
					
					epcTagReads.add(epcTag);
				}
			} else if (payload instanceof PassiveRFIDTagReadPayload) {
				// single tag read
				PassiveRFIDTagRead tagread = (PassiveRFIDTagRead)((PassiveRFIDTagReadPayload)payload).getTagRead();
				EPCTag epcTag = (EPCTag) tagread.getTag();
				
				if (logger.isTraceEnabled()) {
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "collectTagReads", "epcTag=" + epcTag);
				}
				
				epcTagReads.add(epcTag);
			} else {
				// assume a generic, map-based payload
				String eventType = event.getHeader().getEventType();
				IGenericGroup group = payload.getGroup(eventType);
				String[] tagArray = group.getStringAttributeArrayValue(ValidationRuc.TAG_LIST);
				
				if (tagArray != null && tagArray.length > 0) {
					EPCTag epcTag = null;
					for (int i=0; i < tagArray.length; i++) {
						epcTag = (EPCTag) EPCTag.getInstance();
						epcTag.setTagID(tagArray[i]);
						epcTagReads.add(epcTag);
					}
				} else {									
					// if empty, look for single tag
					String tag = group.getStringAttributeValue(ValidationRuc.TAG_URN);
					EPCTag epcTag = (EPCTag) EPCTag.getInstance();
					epcTag.setTagID(tag);
					epcTagReads.add(epcTag);
				}
			}
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.tagreadagent.tagread.count"), epcTagReads.size());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "collectTagReads", message);
			}
			
			return epcTagReads;
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "collectTagReads", e);
			e.printStackTrace();
			throw new TaskAgentException(e);
		} finally {
			logger.traceExit(this, "collectTagReads");
		}
	}
	
	/**
	 * Adds additional info from all EPCTags to the event's payload metadata
	 * @param assetCache
	 * @param event
	 * @param epcTags
	 * @return
	 * @throws TaskAgentException
	 */
	private ISensorEvent addExtensions(Map<String, String> assetCache, ISensorEvent event, Collection<EPCTag> epcTags) throws TaskAgentException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "addExtensions");
			
			if (event == null || epcTags == null) {
				return event;
			}
			
			EPCTag epcTag = null;
			for (Iterator<EPCTag> i=epcTags.iterator(); i.hasNext(); ) {
				epcTag = i.next();
				this.addExtension(assetCache, event, epcTag);
			}
			
			return event;
		} finally {
			logger.traceExit(this, "addExtensions");
		}
	}
	
	/**
	 * Queries the asset description for a given tag URI
	 * @param assetCache
	 * @param tagUri
	 * @return
	 * @throws TaskAgentException
	 */
	private String queryAssetDescrition(Map<String, String> assetCache, String tagUri) throws TaskAgentException {
		int epcAssetType = this.extractEpcAssetType(tagUri);
		String description = null;
		if (assetCache.containsKey(String.valueOf(epcAssetType))) {
			description = assetCache.get(String.valueOf(epcAssetType));
		} else {
			description = this.findAssetDescription(assetCache, epcAssetType);
		}
		
		return description;
	}
	
	/**
	 * Adds additional info from the EPCTag to the event's payload metadata
	 * @param assetCache
	 * @param event
	 * @param epcTag
	 * @return
	 * @throws TaskAgentException
	 */
	private ISensorEvent addExtension(Map<String, String> assetCache, ISensorEvent event, EPCTag epcTag) throws TaskAgentException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "addExtension");
			
			if (event == null || epcTag == null) {
				return event;
			}
			
			String tagUri = epcTag.getAsTagURI();
			String tagId = epcTag.getTagID();
			String idUri = epcTag.getAsIDURI();
			String rawUri = epcTag.getAsRawURI();
			
			if (logger.isTraceEnabled()) {
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "addExtension", "epcTag=" + epcTag);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "addExtension", "tagUri=" + tagUri);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "addExtension", "tagId=" + tagId);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "addExtension", "rawUri=" + rawUri);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "addExtension", "idUri=" + idUri);
			}
			
			// query the asset's description
			String description = this.queryAssetDescrition(assetCache, tagUri);
			
			// update the payload metadata
			IPayloadMetaData pmd = event.getPayloadMetaData();
			
			String name = ReportingRuc.EPCIS_EXT + "." + ReportingRuc.EXT_TAG_ID + "." + tagUri.replaceAll(":", "_");
			String value = tagId;
			pmd.addStringAttribute(name, value);
			
			name = ReportingRuc.EPCIS_EXT + "." + ReportingRuc.EXT_DESCRIPTION + "." + tagUri.replaceAll(":", "_");
			value = description;
			pmd.addStringAttribute(name, value);
			
			return event;
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "addExtension", e);
			e.printStackTrace();
			throw new TaskAgentException(e);
		} finally {
			logger.traceExit(this, "addExtension");
		}
	}
	
	/**
	 * Extract the EPC asset type from the tag URN
	 * @param tagUrn
	 * @throws Exception
	 */
	private int extractEpcAssetType(String tagUrn) throws TaskAgentException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "extractEpcAssetType");
			
			PremisesSessionFacadeLocal premises = premisesHome.create();
			
			int epcAssetType = -1;
			
			if (tagUrn.startsWith(Utils.GRAI_96)) {
				// grai-96
				
				// extract the asset type
				String [] tokens = tagUrn.split("\\.");
				String assetType = tokens[2];
				epcAssetType = Integer.parseInt(assetType);
			} else {
				// other tag types
				
				// extract raw hex value
				String rawUri = null;
				if (tagUrn.startsWith(Utils.RAW_96)) {
					rawUri = tagUrn.substring(Utils.RAW_96.length());
				} else {
					rawUri = premises.toEpcRawURI(tagUrn);
				}
				
				if (logger.isTraceEnabled()) {
					String message = utils.formatMessage(rb.getString("ibmct.tagreadagent.epc.raw.uri"), String.valueOf(rawUri));
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "extractEpcAssetType", message);
				}
			
				if (rawUri != null) {
					// convert to binary
					StringBuffer binary = new StringBuffer();
					for (int b=0; b<rawUri.length(); b++) {
						int digit = Character.digit(rawUri.charAt(b), 16);
						
						StringBuffer binDigits = new StringBuffer(Integer.toBinaryString(digit));
						if (binDigits.length() < 4) {
							int padCount = 4 - binDigits.length();
							if (padCount > 0) {
								for (int p=0; p<padCount; p++) {
									binDigits.insert(0, "0");
								}
							}
						}
						
						binary.append(binDigits);
					}
					
					if (logger.isTraceEnabled()) {
						String message = utils.formatMessage(rb.getString("ibmct.tagreadagent.epc.binary"), binary.toString());
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "extractEpcAssetType", message);
					}
					
					// extract the asset type
					String assetType = binary.toString().substring(34, 58);
					
					if (logger.isTraceEnabled()) {
						String message = utils.formatMessage(rb.getString("ibmct.tagreadagent.epc.asset.type"), assetType);
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "extractEpcAssetType", message);
					}
				
					// convert to decimal
					epcAssetType = Integer.valueOf(assetType, 2);
					
					if (logger.isTraceEnabled()) {
						String message = utils.formatMessage(rb.getString("ibmct.tagreadagent.decimal.asset.type"), String.valueOf(epcAssetType));
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "extractEpcAssetType", message);
					}
				}
			}
			
			return epcAssetType;
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "extractEpcAssetType", e);
			e.printStackTrace();
			throw new TaskAgentException(e);
		} finally {
			logger.traceExit(this, "extractEpcAssetType");
		}
	}
	
	/**
	 * Finds the description for the asset with the given EPC asset type
	 * @param assetCache
	 * @param epcAssetType
	 * @return
	 * @throws TaskAgentException
	 */
	private String findAssetDescription(Map<String, String> assetCache, int epcAssetType) throws TaskAgentException {
		Logger logger = utils.logger;
		
		try {
			String description = "";
			
			if (epcAssetType != -1) {
				try {
					JdbcSessionFacadeLocal jdbc = jdbcHome.create();
					List<Asset> assetsForEpcAssetType = jdbc.findAllAssetsForEpcAssetType(epcAssetType);
					if (assetsForEpcAssetType != null && assetsForEpcAssetType.size() > 0) {
						Asset asset = assetsForEpcAssetType.get(0);
						description = asset.getDescription();
						assetCache.put(String.valueOf(epcAssetType), description);
					}
				} catch (Exception e) {
					description = "";
				}
			}

			return description;
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "extractEpcAssetType", e);
			e.printStackTrace();
			throw new TaskAgentException(e);
		} finally {
			logger.traceExit(this, "extractEpcAssetType");
		}
	}
	
	/**
	 * onMessage
	 */
	public void onMessage(javax.jms.Message msg) {
		super.onMessage(msg);
	}

	/**
	 * getMessageDrivenContext
	 */
	public javax.ejb.MessageDrivenContext getMessageDrivenContext() {
		return fMessageDrivenCtx;
	}

	/**
	 * setMessageDrivenContext
	 */
	public void setMessageDrivenContext(javax.ejb.MessageDrivenContext ctx) {
		fMessageDrivenCtx = ctx;
	}

	/**
	 * ejbCreate
	 */
	public void ejbCreate() {
		utils = Utils.singleton();
		rb = ResourceBundle.getBundle("com.ibm.premises.ct.ejb.resources.TaskAgentMessages");
		
		Logger logger = utils.logger;
		
		try {
			rucHome = (RucSessionFacadeLocalHome) utils.locator.getEJBLocalHome(Utils.RUC_SESSION_FACADE_REF);
			jdbcHome = (JdbcSessionFacadeLocalHome) utils.locator.getEJBLocalHome(Utils.JDBC_SESSION_FACADE_REF);
			premisesHome = (PremisesSessionFacadeLocalHome) utils.locator.getEJBLocalHome(Utils.PREMISES_SESSION_FACADE_REF);
			
			logger.traceEntry(this, "ejbCreate");
		} catch (Exception e) {
			logger.traceException(Logger.MESSAGE_TYPE_ERROR, this, "ejbCreate", e);
			e.printStackTrace();
			throw new TaskAgentRuntimeException(e.getMessage());
		} finally {
			logger.traceExit(this, "ejbCreate");
		}
	}

	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}

}
