/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets.actions;

import java.util.Map;
import java.util.ResourceBundle;

import com.ibm.premises.ct.dto.Asset;
import com.ibm.premises.ct.dto.PremisesPrinter;
import com.ibm.premises.ct.ejb.JdbcSessionFacadeLocal;
import com.ibm.premises.ct.ejb.JdbcSessionFacadeLocalHome;
import com.ibm.premises.ct.ejb.PremisesSessionFacadeLocal;
import com.ibm.premises.ct.ejb.PremisesSessionFacadeLocalHome;
import com.ibm.premises.ct.ejb.RucSessionFacadeLocal;
import com.ibm.premises.ct.ejb.RucSessionFacadeLocalHome;
import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.premises.ct.utils.Utils;
import com.ibm.rfid.premises.logger.Logger;
import com.ibm.sensorevent.uuid.UUID;

/**
 * Abstract action
 */
public abstract class Action implements IAction {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	protected static Utils utils;
	protected static ResourceBundle rb;
	protected static RucSessionFacadeLocalHome rucHome;
	protected static JdbcSessionFacadeLocalHome jdbcHome;
	protected static PremisesSessionFacadeLocalHome premisesHome;
	
	protected static final String SERVLET_URL = "/ibmct/UIActionHandler";
	protected static final String JDBC_SESSION_FACADE_REF = Utils.JDBC_SESSION_FACADE_REF;
	protected static final String RUC_SESSION_FACADE_REF = Utils.RUC_SESSION_FACADE_REF;
	protected static final String PREMISES_SESSION_FACADE_REF = Utils.PREMISES_SESSION_FACADE_REF;
	protected static final String PROP_CT_TEST = "com.ibm.premises.ct.test";
	protected static final String PROP_CT_TEST_TRUE = "true";
	protected static final String PROP_CT_TEST_DEFAULT = "false";
	
	protected static final String GRAI_96 = Utils.GRAI_96;
	protected static final String RAW_96 = Utils.RAW_96;
	
	protected static final String EPC_DECODE_ASSET_TYPE_KEY = "AssetType";
	
	public static final String ACTION_ID = "action";
	
	public static final String JSON_STATUS_MESSAGE =
		"'{'\"statusMessage\": \"{0}\"'}'";
	public static final String JSON_ERROR_MESSAGE =
		"'{'\"errorMessage\": \"{0}\"'}'";
	
	public Action() throws Exception {
		utils = Utils.singleton();
		rb = ResourceBundle.getBundle("com.ibm.premises.ct.ui.resources.Messages");
		rucHome = (RucSessionFacadeLocalHome) utils.locator.getEJBLocalHome(RUC_SESSION_FACADE_REF);
		jdbcHome = (JdbcSessionFacadeLocalHome) utils.locator.getEJBLocalHome(JDBC_SESSION_FACADE_REF);
		premisesHome = (PremisesSessionFacadeLocalHome) utils.locator.getEJBLocalHome(PREMISES_SESSION_FACADE_REF);
	}
	
	public String toString() {
		return this.getClass().getSimpleName();
	}
	
	protected String getDefaultActionId() throws ActionException {
		return this.getClass().getSimpleName();
	}
	
	protected String createJsonStatusMessage(String s) {
		return utils.formatMessage(JSON_STATUS_MESSAGE, utils.escape(s));
	}
	
	protected String createJsonErrorMessage(String s) {
		return utils.formatMessage(JSON_ERROR_MESSAGE, utils.escape(s));
	}
	
	protected String getPremisesProperty(String key) throws Exception {
		Logger logger = utils.logger;
		try {
			logger.traceEntry(this, "getPremisesProperty");
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.get.property"), key);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getPremisesProperty", message);
			}
			
			PremisesSessionFacadeLocal local = premisesHome.create();
			
			String value = local.getPremisesProperty(key);
			
			return value;			
		} finally {
			logger.traceExit(this, "getPremisesProperty");
		}
	}
	
	protected String getPremisesProperty(String key, String defaultValue) throws Exception {
		Logger logger = utils.logger;
		try {
			logger.traceEntry(this, "getPremisesProperty");
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.get.property.default"), key, defaultValue);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getPremisesProperty", message);
			}
			
			PremisesSessionFacadeLocal local = premisesHome.create();
			
			String value = local.getPremisesProperty(key, defaultValue);
			
			return value;			
		} finally {
			logger.traceExit(this, "getPremisesProperty");
		}
	}
	
	protected PremisesPrinter getPrinter(String id) throws Exception {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "getPrinter");
			
			// query the printers
			PremisesSessionFacadeLocal local = premisesHome.create();
			PremisesPrinter printer = local.getPrinter(id);
			
			return printer;			
		} finally {
			logger.traceExit(this, "getPrinter");
		}
	}
	
	protected Asset getAsset(String id) throws Exception {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "getAsset");
			
			// query the asset type
			JdbcSessionFacadeLocal jdbcLocal = jdbcHome.create();
			Asset asset = jdbcLocal.findAsset(id);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.asset"), 
						new Object[] {id, asset.getDescription(), String.valueOf(asset.getEpcAssetType()), asset.getEpcPackType(), asset.getLabelData().toString()});
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "getAsset", message);
			}
			
			return asset;			
		} finally {
			logger.traceExit(this, "getAsset");
		}
	}

	protected boolean isTestMode() throws Exception {
		String sIsTest = this.getPremisesProperty(PROP_CT_TEST, PROP_CT_TEST_DEFAULT);
		boolean isTest = PROP_CT_TEST_TRUE.equals(sIsTest);
		return isTest;
	}
	
	protected String createUUID() {
		String uuid = UUID.createUUID();
		return uuid;
	}

	@SuppressWarnings("unchecked")
	protected Map<String, String> epcDecodeRawUri(String rawUri) throws Exception {
		RucSessionFacadeLocal ruc = rucHome.create();
		Map<String, String> decodedEpc = ruc.epcDecode(rawUri);
		return decodedEpc;
	}
}
