/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets.actions;

import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.premises.ct.dto.Asset;
import com.ibm.premises.ct.dto.PremisesProfile;
import com.ibm.premises.ct.ejb.JdbcSessionFacadeLocal;
import com.ibm.premises.ct.ejb.PremisesSessionFacadeLocal;
import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Get asset types action
 */
public class GetAssetTypesAction extends Action {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	public static final String EPC_PROFILE = "epc_profile";
	
	public static final String JSON_TEXT = 
		"'{'\"assetTypes\": [\n" +
			"{0}\n" +
		"]'}'";
	public static final String JSON_ELEMENT =
		"'{'\"id\": \"{0}\", \"description\": \"{1}\", \"hidden\": \"{2}\"'}'";
	
	public GetAssetTypesAction() throws Exception {
		super();
	}

	public String handleAction(HttpServletRequest request, HttpServletResponse response) throws ActionException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "handleAction");
			
			// query the parameters
			String epcProfile = request.getParameter(EPC_PROFILE);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), EPC_PROFILE, epcProfile);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			if (epcProfile == null) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), EPC_PROFILE);
				throw new ActionException(message);
			}
			
			// query the Premises profile
			PremisesSessionFacadeLocal premisesEjb = premisesHome.create();
			PremisesProfile premisesProfile = premisesEjb.getProfile(epcProfile);
			
			if (premisesProfile == null) {
				if (utils.logger.isTraceEnabled()) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.missing.epc.profile"), epcProfile);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				}
				
				String message  = utils.escape(rb.getString("ibmct.ui.trace.error"));
				
				// create the JSON text
				String jsonText = this.createJsonStatusMessage(message);
				
				if (logger.isTraceEnabled()) {
					message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				}
				
				return jsonText;
			}
			
			// query the asset types
			// sort them by description (as defined in the Asset.compareTo method)
			JdbcSessionFacadeLocal jdbcEjb = jdbcHome.create();

			// only include the assets that have a corresponding pack type in the profile
			SortedSet<Asset> assets = new TreeSet<Asset>();
			for (Iterator<Asset> i=jdbcEjb.findAllAssets().iterator(); i.hasNext(); ) {
				Asset asset = i.next();
				if (premisesProfile.getPackTypeNames().contains(asset.getEpcPackType())) {
					assets.add(asset);
				}
			}
			
			// create the JSON text
			StringBuffer jsonAssetTypes = new StringBuffer();
			
			for (Iterator<Asset> i=assets.iterator(); i.hasNext(); ) {
				Asset asset = i.next();
				String id = asset.getId();
				String description = asset.getDescription();
				boolean hidden = asset.isHidden();
				
				jsonAssetTypes.append(utils.formatMessage(JSON_ELEMENT, utils.escape(id), utils.escape(description), utils.escape(String.valueOf(hidden))));
				
				if (i.hasNext()) {
					jsonAssetTypes.append(",\n");
				}
			}
			
			String jsonText = utils.formatMessage(JSON_TEXT, jsonAssetTypes.toString());
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			return jsonText;

		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
			throw new ActionException(e);
		} finally {
			logger.traceExit(this, "handleAction");
		}
	}
	
}
