/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.SortedSet;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.premises.eventmonitor.ejb.EventMonitorSession;
import com.ibm.premises.eventmonitor.ejb.EventMonitorSessionHome;
import com.ibm.rfid.premises.logger.Logger;
import com.ibm.sensorevent.model.IGroup;
import com.ibm.sensorevent.model.IPayload;
import com.ibm.sensorevent.model.ISensorEvent;
import com.ibm.sensorevent.model.payload.ISensorEventPayload;
import com.ibm.sensorevent.util.Constants;

/**
 * Get events action
 */
public class GetEventsAction extends Action {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	protected static EventMonitorSessionHome monitorHome;
	
	public static final String ALERT_ERROR = Constants.Alert_Error_Event_Name;
	public static final String ALERT_WARNING = Constants.Alert_Warning_Event_Name;
	public static final String ALERT_INFO = Constants.Alert_Info_Event_Name;
	public static final String ALERT_DEBUG = Constants.Alert_Debug_Event_Name;
	public static final String MESSAGE_SELECTOR = 
		"ibmse='" + ALERT_ERROR + "' or " +
		"ibmse='" + ALERT_WARNING + "' or " +
		"ibmse='" + ALERT_INFO + "' or " +
		"ibmse='" + ALERT_DEBUG + "'";

	public static final String JSON_TEXT =
		"'{'\"events\": [\n" + 
			"{0}\n" +
		"]'}'";
	public static final String JSON_ELEMENT =
		"'{'\"message\": \"{0}\"'}'";
	
	public GetEventsAction() throws Exception {
		super();
		monitorHome = (EventMonitorSessionHome) utils.locator.getEJBHome("ejb/com/ibm/premises/eventmonitor/ejb/EventMonitorSessionHome", EventMonitorSessionHome.class);
	}

	public String handleAction(HttpServletRequest request, HttpServletResponse response) throws ActionException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "handleAction");
			
			EventMonitorSession monitor = monitorHome.create();
			
			List<String> statusMessages = new ArrayList<String>();

			ISensorEvent event = null;
			
			SortedSet events = monitor.getEvents(MESSAGE_SELECTOR);
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.action.get_event_action.cache.size"), events.size());
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
					
			for (Iterator i=events.iterator(); i.hasNext(); ) {
				event = (ISensorEvent) i.next();
				String eventType = event.getHeader().getEventType();
				
				if (logger.isTraceEnabled()) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.action.get_event_action.eventtype"), eventType);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				}
				
				// get the event dateTime
				String dateTime = event.getHeader().getDateTimeAsString();
				
				// get the alert message
				try {
					IPayload payload = event.getPayload();
					if (payload instanceof ISensorEventPayload) {
						ISensorEventPayload sep = (ISensorEventPayload) payload;
						IGroup eventGroup = sep.getEventGroup();
						String alertMessage = eventGroup.getStringAttributeValue("value");
						
						// construct the status message
						String alertMessageText = null;
						String cssClassName = null;
						
						if (ALERT_ERROR.equals(eventType)) {
							cssClassName = "statusAlertErrorEvent";
							alertMessageText = utils.formatMessage(rb.getString("ibmct.ui.status.alert.error"), dateTime, alertMessage);
						} else if (ALERT_WARNING.equals(eventType)) {
							cssClassName = "statusAlertWarningEvent";
							alertMessageText = utils.formatMessage(rb.getString("ibmct.ui.status.alert.warning"), dateTime, alertMessage);
						} else if (ALERT_INFO.equals(eventType)) {
							cssClassName = "statusAlertInfoEvent";
							alertMessageText = utils.formatMessage(rb.getString("ibmct.ui.status.alert.info"), dateTime, alertMessage);
						} else {
							// debug
							cssClassName = "statusAlertDebugEvent";
							alertMessageText = utils.formatMessage(rb.getString("ibmct.ui.status.alert.debug"), dateTime, alertMessage);
						}
						
						String eventMessage = "<P class=\"" + cssClassName + "\">" + alertMessageText + "</P>";

						if (logger.isTraceEnabled()) {
							String message = utils.formatMessage(rb.getString("ibmct.ui.action.get_event_action.event.message"), eventMessage);
							logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
						}

						statusMessages.add(eventMessage);
					} else {
						if (logger.isTraceEnabled()) {
							String message = utils.formatMessage(rb.getString("ibmct.ui.action.get_event_action.skipping.payload"), payload.getClass().getName());
							logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
						}
					}
				} catch (Exception e) {
					if (logger.isTraceEnabled()) {
						logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
					}
					
					continue;
				}
			}
			
			// create the JSON text
			StringBuffer jsonStatusMessages = new StringBuffer();
			
			for (Iterator<String> i=statusMessages.iterator(); i.hasNext(); ) {
				String s = i.next();
				jsonStatusMessages.append(utils.formatMessage(JSON_ELEMENT, utils.escape(s)));
				
				if (i.hasNext()) {
					jsonStatusMessages.append(",\n");
				}
			}
			
			String jsonText = utils.formatMessage(JSON_TEXT, jsonStatusMessages.toString());
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			return jsonText;
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
			throw new ActionException(e);
		} finally {
			logger.traceExit(this, "handleAction");
		}
	}
	
}
