/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */

package com.ibm.premises.ct.servlets.actions;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.premises.ct.dto.Asset;
import com.ibm.premises.ct.dto.PremisesPrinter;
import com.ibm.premises.ct.ejb.JdbcSessionFacadeLocal;
import com.ibm.premises.ct.exceptions.ActionException;
import com.ibm.premises.ct.servlets.ReprintThread;
import com.ibm.rfid.premises.logger.Logger;

/**
 * Reprint action
 */
public class ReprintAction extends Action {
	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
			+ "All Rights Reserved * Licensed Materials - Property of IBM "
			+ "US Government Users Restricted Rights - Use, duplication or "
			+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	
	public static final String PRINTER = "reprint_printer";
	public static final String EPC_PROFILE = "reprint_epc_profile";
	public static final String TAGID = "reprint_tagid";
	
	public static final String PROPERTY_PRINTER_MAX_COUNT = "com.ibm.premises.ct.printer.max.count";
	public static final String PROPERTY_PRINTER_MAX_COUNT_DEFAULT = "5000";
	
	public static final String UI_PRINTER_OPTIONS = "reprint_printer_options";
	public static final String UI_EPC_PROFILE_OPTIONS = "reprint_epc_profile_options";
	public static final String UI_SUBMIT = "reprint_button";
	
	public ReprintAction() throws Exception {
		super();
	}

	public String handleAction(HttpServletRequest request, HttpServletResponse response) throws ActionException {
		Logger logger = utils.logger;
		
		try {
			logger.traceEntry(this, "handleAction");
			
			// query the parameters
			String printer = request.getParameter(PRINTER);
			String epcProfile = request.getParameter(EPC_PROFILE);
			String tagId = request.getParameter(TAGID);
			
			if (logger.isTraceEnabled()) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), PRINTER, printer);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), EPC_PROFILE, epcProfile);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.parameter"), TAGID, tagId);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			if (printer == null || printer.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), PRINTER);
				throw new ActionException(message);
			}
			
			if (epcProfile == null || epcProfile.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), EPC_PROFILE);
				throw new ActionException(message);
			}
			
			if (tagId == null || tagId.length() == 0) {
				String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.null.parameter"), TAGID);
				throw new ActionException(message);
			}
			
			// extract the assetType from the tagId
			Map<String, String> decodedEpc = this.epcDecodeRawUri(tagId);
			String assetId = null;
			if (decodedEpc != null && decodedEpc.containsKey(EPC_DECODE_ASSET_TYPE_KEY)) {
				String epcAssetType = decodedEpc.get(EPC_DECODE_ASSET_TYPE_KEY);
				
				// find the asset with this epcAssetType
				JdbcSessionFacadeLocal jdbc = jdbcHome.create();
				List<Asset> assets = jdbc.findAllAssetsForEpcAssetType(Integer.parseInt(epcAssetType));
				if (assets != null && assets.size() > 0) {
					Asset asset = assets.get(0);
					assetId = asset.getId();
				} else {
					if (logger.isTraceEnabled()) {
						String message = utils.formatMessage(rb.getString("ibmct.ui.trace.warning.handler.reprint.no.assets.for.type"), epcAssetType);
						logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
					}
				}
			} else {
				if (logger.isTraceEnabled()) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.warning.handler.reprint.no.epc.decode"), tagId);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				}
			}
			
			// query the asset type info
			Asset asset = this.getAsset(assetId);
			if (asset == null) {
				if (logger.isTraceEnabled()) {
					String message = utils.formatMessage(rb.getString("ibmct.ui.trace.error.handler.reprint.asset.not.found"), tagId);
					logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
				}
			}
			
			// are we running in test mode?
			boolean isTest = this.isTestMode();
			
			if (!isTest) {
				// reprint the tag
				ReprintThread thread = new ReprintThread(rucHome, printer, epcProfile, asset.getEpcPackType(), tagId, asset.getLabelData());
				thread.start();
			}
			
			// publish the info alert
			PremisesPrinter premisesPrinter = this.getPrinter(printer);
			String message = utils.formatMessage(rb.getString("ibmct.ui.reprint.complete.alert"), premisesPrinter.getDescription());
			utils.publishInfoEvent(message);
			
			// create the JSON text
			String jsonText = this.createJsonStatusMessage(rb.getString("ibmct.ui.reprint.complete"));
			
			if (logger.isTraceEnabled()) {
				message = utils.formatMessage(rb.getString("ibmct.ui.trace.info.handler.json"), jsonText);
				logger.trace(Logger.TRACE_TYPE_DEBUG, this, "handleAction", message);
			}
			
			return jsonText;
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.traceException(Logger.TRACE_TYPE_DEBUG, this, "handleAction", e);
			throw new ActionException(e);			
		} finally {
			logger.traceExit(this, "handleAction");
		}
	}
	
}
